require 'httparty'

module Telefacil
  class Base
    include HTTParty

    # API BASE URLS
    API_BASE     = 'https://scgi.duocom.es/cgi-bin/telefacil2/'
    API_CANARIAS = 'https://cgi.duocom.es/cgi-bin/telefacil2/'

    def self.configurar
      yield(self)
    end

    def self.api_canarias
      @@api_canarias ||= nil
    end

    def self.api_canarias=( api_canarias )
      @@api_canarias = api_canarias
    end

    def self.principal
      @@principal ||= nil
    end

    def self.password
      @@password ||= nil
    end

    def self.principal=( principal )
      @@principal = principal
    end

    def self.password=( password )
      @@password = password
    end

    def self.autenticado?
      (principal.nil? || password.nil?) ? false : true
    end

    private

      def self.request(params={}, accion=nil)
        raise Telefacil::AutenticacionError unless autenticado?
        opts  = construir_opts_query(params, accion)
        url   = construir_url
        respuesta = http_response(url, opts)
        respuesta_parseada = parse(respuesta)
      rescue StandardError => e
        puts 'Telefacil Error: '+ e.message
        false
      end

      def self.api_endpoint(api_endpoint = nil)
        @_api_endpoint ||= api_endpoint
      end

      def self.api_model(api_model = nil)
        @_api_model ||= api_model
      end

      def self.api_singular(api_singular = nil)
        @_api_singular ||= api_singular
      end

      def self.construir_opts_query(params, accion)
        opts = {
          :query => {
            :principal => principal,
            :pass => password,
            :tipo_salida => 'xml'
          }
        }
        opts[:query][:accion] = accion if accion
        opts[:query].merge!(params)    if !params.empty?
        opts
      end

      def self.construir_url
        (api_canarias.nil? ? API_BASE : API_CANARIAS) + self.api_endpoint
      end

      def self.http_response(url, opts)
        respuesta = HTTParty.get(url, opts)
        xml      = respuesta.parsed_response
        parsear_a_os(xml)
      end

      def self.parsear_a_os(xml)
        JSON.parse(xml.to_json, object_class: OpenStruct)
      end

      def self.parse(datos)
        d = datos.send("#{plural(self.api_model)}")

        hay_resultado = (d.res) && (d.res == "1")
        no_hay_error  = (d.error) && (d.error == "0")

        if (hay_resultado)||(no_hay_error)
          respuesta = d.send(atributo_modelo)
          respuesta = convertir_a_array(respuesta)
          Exito.new(respuesta)
        else
          Error.new(d.msg)
        end
      end

      def self.atributo_modelo
        self.api_singular.nil? ? "#{self.api_model}" : "#{self.api_singular}"
      end

      def self.plural(str)
        self.api_model + "s"
      end

      def self.convertir_a_array(respuesta)
        if !respuesta.is_a?(Array)
          respuesta = [respuesta].compact
        end
        respuesta
      end
  end
end
