module Telefacil
  class Extension < Base

    api_endpoint   'api_desvios'
    api_model      'desvio'

    # PARÁMETROS
    #   'ext': Número de extensión a crear.
    # 
    #   Opcional:
    #   'pin_ext': Un PIN de 4 dígitos para esa extensión (opcional).
    #   'nombre': Nombre de la extensión (opcional).
    #   'secuencia': para que la extensión sea de tipo secuencial (opcional). Ejemplo: 1_234_7_889

    def self.crear(opts={})
      request(opts, 'crear_ext')
    end

    def self.borrar(opts={})
      request(opts, 'borrar_ext')
    end

    # PARÁMETROS:
    #   nv' o 'ext': Número virtual o de extensión cuyo buzón se va a configurar.
    #   'valor': puede ser
    #     - Un número entre 1 y 60 segundos (segundos para el desvío antes de enviar al buzón), o
    #     - 'solo_buzon' (no realizar desvío y mandar directamente la llamada al buzón), o
    #     - 'solo_desvio' (desactivar el buzón de voz)
    #     - 'salto_a_extAAA' (donde AAA es el número de extensión a la que se salta) o 'salto_a_centralita'. Esta opción solo sirve para extensiones. Se puede añadir entonces el parámetro opcional 'tiempo_para_salto', que puede estar entre 1 y 60, y que especifica el número se segundos que se realiza el desvío antes de saltar a otra extensión.
    #
    #   Opcionalmente, se puede añadir el parámetro no_grabar=1, que hace sonar sólo el mensaje de buzón pero no graba el mensaje del llamante.

    def self.post_desvio(opts={})
      request(opts, 'post')
    end

    # PARÁMETROS
    #   'ext': Número virtual o de extensión cuyo buzón se va a configurar.
    #   'mascara': el número de teléfono que le llegará a los desvíos. Puede ser 'privado' (número oculto), 'semiprivado' (parcialmente oculto por ceros), 'numero_virtual', 'centralita' o 'centralita3digitos' (número de centralita seguido de 3 dígitos especificados en el parámetro 'digitos').
    #   'reduccion_espera': si este valor es 1, se activará la opción avanzada "Reducir tiempos de espera si un desvío comunica".

    def self.configurar_mascara_y_reduccion_esperas(params={})
      ext     = params.fetch(:ext, nil)
      mascara = params.fetch(:mascara, "centralita3digitos")
      digitos = params.fetch(:digitos, nil)
      reduccion_espera = params.fetch(:reduccion_espera, "1")

      opts = { :ext => ext, :reduccion_espera => reduccion_espera, :mascara => mascara, :digitos => digitos }
      request(opts, 'cambiar')
    end

  end
end
